<?php
	
	/*
		Source: https://github.com/damog/planetalinux/blob/master/www/principal/suscripcion/lib/antiflood.hack.php
		and		https://stackoverflow.com/questions/12553606/anti-flood-ddos-in-php
		
		*	Antiflood - Generic antiflood protection
		*	------------------------------------------------------------------------
		*	Copyright (C) 2004-2006, J. Carlos Nieto <xiam@users.sourceforge.net>
		*	This program is Free Software.
		*
		*	@license	http://www.gnu.org/copyleft/gpl.html GNU/GPL License 2.0
		*	@author		J. Carlos Nieto <xiam@users.sourceforge.net>
		*	@version	CVS: $Id$
		*	@link		http://xiam.menteslibres.org
		
		I'm using a good anti-flood script that does not need cookies (perfect for webservices). 
		It's not perfect against advanced DDOS attacks but it's enough for preventing beginners attacks and automatic multiple requests.
		
		
		*	2019-08-01 adjusted by Olaf Nöhring, https://datenbank-projekt.de
		*		added antiflood_countaccess(); at bottom to automatically count access when this file is being included.
		*		added timer script to visualize when the user might retry.
		*		added some documentation how to use this in AppGini
		*	2019-08-20 adjusted by Olaf Nöhring, https://datenbank-projekt.de
		*		changed include position to deny browser back function (Thanks to AhmedBR, https://forums.appgini.com/phpbb/posting.php?mode=reply&f=4&t=3168#pr10834)

		------------------------
		- General instructions -
		------------------------
			How to use?
			Create a "antiflood" directory, it must be writable for http server
			
			include this script on the page that you want to protect:
			
			<?php include("antiflood.php"); ?>
		
		--------------------------
		- For the use in AppGini -
		--------------------------
			1. Place this file in the hooks folder (hooks/antiflood.php).
			2. Create a subfolder "antiflood" in your hooks folder (hooks/antiflood), make sure folder is writeable by webserver (750)
			3. Include this file
				/login.php as the very first line. Simply add 
					<?php include("$currDir/hooks/antiflood.php"); ?>
				before everything else in that file.
			
			Important: Please note, that the login.php will be overwritten, once you regenerate your files with AppGini.
			
			
		------------------------
		- Adjust to your needs -
		------------------------
			You may adjust 
				// number of allowed page requests for the user
				// time interval to start counting page requests (seconds)
				// seconds to punish the user who has exceeded in doing requests
				// writable directory to keep script data 
			below.
	*/
	
	//Olaf: removed this line ... why was it there?
	//	if (!defined("SAFE_INCLUDE")) die("Get a Life!");
	
	define("SCRIPT_ROOT", dirname(__FILE__));
	
	// number of allowed page requests for the user
	define("CONTROL_MAX_REQUESTS", 20);
	
	// time interval to start counting page requests (seconds)
	define("CONTROL_REQ_TIMEOUT", 10);
	
	// seconds to punish the user who has exceeded in doing requests
	define("CONTROL_BAN_TIME", 10*60);
	
	// writable directory to keep script data 
	define("SCRIPT_TMP_DIR", SCRIPT_ROOT."/antiflood");
	
	
	// ##################################################################
	// #####	you don't need to edit below this line				#####
	// ##################################################################
	define("USER_IP", $_SERVER["REMOTE_ADDR"]);
	
	define("CONTROL_DB", SCRIPT_TMP_DIR."/ctrl");
	define("CONTROL_LOCK_DIR", SCRIPT_TMP_DIR."/lock");
	define("CONTROL_LOCK_FILE", CONTROL_LOCK_DIR."/".md5(USER_IP));
	
	@mkdir(CONTROL_LOCK_DIR);
	@mkdir(SCRIPT_TMP_DIR);
	
	if (file_exists(CONTROL_LOCK_FILE)) {
		if (time()-filemtime(CONTROL_LOCK_FILE) > CONTROL_BAN_TIME) {
			// this user has complete his punishment
			unlink(CONTROL_LOCK_FILE);
			} else {
			// too many requests
			echo "<h1>Access denied</h1>";
			echo "As a precaution we have momentarily restricted access to this site. Try again later. <br />";
			echo CONTROL_MAX_REQUESTS . " requests in " . CONTROL_REQ_TIMEOUT . " seconds banned you for ".CONTROL_BAN_TIME." seconds.";
		?>
		<p id="wait"></p>
		<script>
			// Set the date we're counting down to
			var countDownDate = new Date().getTime()+<?php echo 1000*CONTROL_BAN_TIME ?>;		//Date("2019-08-01 08:10:00").getTime();
			
			// Update the count down every 1 second
			var x = setInterval(function() {
				
				// Get today's date and time
				var now = new Date().getTime();
				
				// Find the distance between now and the count down date
				var distance = countDownDate - now;
				
				// Time calculations for days, hours, minutes and seconds
				var days = Math.floor(distance / (1000 * 60 * 60 * 24));
				var hours = Math.floor((distance % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
				var minutes = Math.floor((distance % (1000 * 60 * 60)) / (1000 * 60));
				var seconds = Math.floor((distance % (1000 * 60)) / 1000);
				
				// Output the result in an element with id="wait"
				document.getElementById("wait").innerHTML = "Wait for " + days + "d " + hours + "h "
				+ minutes + "m " + seconds + "s ";
				
				// If the count down is over, write some text 
				if (distance < 0) {
					clearInterval(x);
					document.getElementById("wait").innerHTML = "You may <a href='index.php?signIn=1'>retry now</a>.";
				}
			}, 1000);
		</script>				
		<?php			
			touch(CONTROL_LOCK_FILE);
			die;
		}
	}
	
	function antiflood_countaccess() {
		// counting requests and last access time
		$control = Array();
		
		if (file_exists(CONTROL_DB)) {
			$fh = fopen(CONTROL_DB, "r");
			$control = array_merge($control, unserialize(fread($fh, filesize(CONTROL_DB))));
			fclose($fh);
		}
		
		if (isset($control[USER_IP])) {
			if (time()-$control[USER_IP]["t"] < CONTROL_REQ_TIMEOUT) {
				$control[USER_IP]["c"]++;
				} else {
				$control[USER_IP]["c"] = 1;
			}
			} else {
			$control[USER_IP]["c"] = 1;
		}
		$control[USER_IP]["t"] = time();
		
		if ($control[USER_IP]["c"] >= CONTROL_MAX_REQUESTS) {
			// this user did too many requests within a very short period of time
			$fh = fopen(CONTROL_LOCK_FILE, "w");
			fwrite($fh, USER_IP);
			fclose($fh);
		}
		
		// writing updated control table
		$fh = fopen(CONTROL_DB, "w");
		fwrite($fh, serialize($control));
		fclose($fh);
	}
	
	antiflood_countaccess();
?>